<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2008 Stanislas Rolland <stanislas.rolland(arobas)fructifor.ca>
*  All rights reserved
*
*  This script is part of the Typo3 project. The Typo3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/
/**
 * Table Operations extension for htmlArea RTE
 *
 * @author Stanislas Rolland <stanislas.rolland(arobas)fructifor.ca>
 *
 * TYPO3 SVN ID: $Id: class.tx_rtehtmlarea_tableoperations.php 3311 2008-02-27 22:54:26Z stan $
 *
 */

require_once(t3lib_extMgm::extPath('rtehtmlarea').'class.tx_rtehtmlareaapi.php');

class tx_rtehtmlarea_tableoperations extends tx_rtehtmlareaapi {

	protected $extensionKey = 'rtehtmlarea';		// The key of the extension that is extending htmlArea RTE
	protected $pluginName = 'TableOperations';		// The name of the plugin registered by the extension
	protected $relativePathToLocallangFile = '';	// Path to this main locallang file of the extension relative to the extension dir.
	protected $relativePathToSkin = 'extensions/TableOperations/skin/htmlarea.css';		// Path to the skin (css) file relative to the extension dir
	protected $htmlAreaRTE;					// Reference to the invoking object
	protected $thisConfig;					// Reference to RTE PageTSConfig
	protected $toolbar;					// Reference to RTE toolbar array
	protected $LOCAL_LANG; 					// Frontend language array
	protected $requiresClassesConfiguration = true;		// True if the registered plugin requires the PageTSConfig Classes configuration
	
	protected $pluginButtons = 'table, toggleborders, tableproperties, tablerestyle, rowproperties, rowinsertabove, rowinsertunder, rowdelete, rowsplit,
						columnproperties, columninsertbefore, columninsertafter, columndelete, columnsplit,
						cellproperties, cellinsertbefore, cellinsertafter, celldelete, cellsplit, cellmerge';
	protected $convertToolbarForHtmlAreaArray = array (
		'table'			=> 'InsertTable',
		'toggleborders'		=> 'TO-toggle-borders',
		'tableproperties'	=> 'TO-table-prop',
		'tablerestyle'		=> 'TO-table-restyle',
		'rowproperties'		=> 'TO-row-prop',
		'rowinsertabove'	=> 'TO-row-insert-above',
		'rowinsertunder'	=> 'TO-row-insert-under',
		'rowdelete'		=> 'TO-row-delete',
		'rowsplit'		=> 'TO-row-split',
		'columnproperties'	=> 'TO-col-prop',
		'columninsertbefore'	=> 'TO-col-insert-before',
		'columninsertafter'	=> 'TO-col-insert-after',
		'columndelete'		=> 'TO-col-delete',
		'columnsplit'		=> 'TO-col-split',
		'cellproperties'	=> 'TO-cell-prop',
		'cellinsertbefore'	=> 'TO-cell-insert-before',
		'cellinsertafter'	=> 'TO-cell-insert-after',
		'celldelete'		=> 'TO-cell-delete',
		'cellsplit'		=> 'TO-cell-split',
		'cellmerge'		=> 'TO-cell-merge',
		);
	
	public function main($parentObject) {
		global $TYPO3_CONF_VARS;
		
		$available = parent::main($parentObject);
		
		if ($this->htmlAreaRTE->client['BROWSER'] == 'opera') {
			$this->thisConfig['hideTableOperationsInToolbar'] = 0;
		}
		
		return $available;
	}
	
	/**
	 * Return JS configuration of the htmlArea plugins registered by the extension
	 *
	 * @param	integer		Relative id of the RTE editing area in the form
	 *
	 * @return 	string		JS configuration for registered plugins, in this case, JS configuration of block elements
	 *
	 * The returned string will be a set of JS instructions defining the configuration that will be provided to the plugin(s)
	 * Each of the instructions should be of the form:
	 * 	RTEarea['.$RTEcounter.']["buttons"]["button-id"]["property"] = "value";
	 */
	public function buildJavascriptConfiguration($RTEcounter) {
		global $TSFE, $LANG;
		
		$registerRTEinJavascriptString = '';
		if (in_array('table', $this->toolbar)) {
			
				// Combining fieldset disablers as a list
			$disabledFieldsets = array('Alignment', 'Borders', 'Color', 'Description', 'Layout', 'RowGroup', 'Spacing', 'Style');
			foreach ($disabledFieldsets as $index => $fieldset) {
				if (!trim($this->thisConfig['disable'.$fieldset.'FieldsetInTableOperations'])) {
					unset($disabledFieldsets[$index]);
				}
			}
			$disabledFieldsets = strtolower(implode(',', $disabledFieldsets));
			
				// Dialogue fieldsets removal configuration
			$dialogues = array('table', 'tableproperties', 'rowproperties', 'columnproperties', 'cellproperties');
			foreach ($dialogues as $dialogue) {
				if (!is_array( $this->thisConfig['buttons.']) || !is_array( $this->thisConfig['buttons.'][$dialogue.'.'])) {
					$registerRTEinJavascriptString .= '
			RTEarea['.$RTEcounter.'].buttons.'.$dialogue.' = new Object();
			RTEarea['.$RTEcounter.'].buttons.'.$dialogue.'.removeFieldsets = "' . $disabledFieldsets . '";';
				} else if ($this->thisConfig['buttons.'][$dialogue.'.']['removeFieldsets']) {
					$registerRTEinJavascriptString .= '
			RTEarea['.$RTEcounter.'].buttons.'.$dialogue.'.removeFieldsets += ",' . $disabledFieldsets . '";';
				} else {
					$registerRTEinJavascriptString .= '
			RTEarea['.$RTEcounter.'].buttons.'.$dialogue.'.removeFieldsets = ",' . $disabledFieldsets . '";';
				}
			}
			
			$registerRTEinJavascriptString .= '
			RTEarea['.$RTEcounter.'].hideTableOperationsInToolbar = ' . (trim($this->thisConfig['hideTableOperationsInToolbar']) ? 'true' : 'false') . ';';
			
				// Deprecated toggleborders button configuration
			if (in_array('toggleborders',$this->toolbar) && $this->thisConfig['keepToggleBordersInToolbar']) {
				if (!is_array($this->thisConfig['buttons.']['toggleborders.'])) {
					$registerRTEinJavascriptString .= '
			RTEarea['.$RTEcounter.'].buttons.toggleborders = new Object();
			RTEarea['.$RTEcounter.'].buttons.toggleborders.keepInToolbar = true;';
				} elseif (!$this->thisConfig['buttons.']['toggleborders.']['keepInToolbar']) {
					$registerRTEinJavascriptString .= '
			RTEarea['.$RTEcounter.'].buttons.toggleborders.keepInToolbar = true;';
				}
			}
		}
		return $registerRTEinJavascriptString;
	}
	
	 
	/**
	 * Return an updated array of toolbar enabled buttons
	 *
	 * @param	array		$show: array of toolbar elements that will be enabled, unless modified here
	 *
	 * @return 	array		toolbar button array, possibly updated
	 */
	public function applyToolbarConstraints($show) {
			// We will not allow any table operations button if the table button is not enabled
		if (!in_array('table', $show)) {
			return array_diff($show, t3lib_div::trimExplode(',', $this->pluginButtons));
		} else {
			return $show;
		}
	}

} // end of class

if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/rtehtmlarea/extensions/TableOperations/class.tx_rtehtmlarea_tableoperations.php']) {
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/rtehtmlarea/extensions/TableOperations/class.tx_rtehtmlarea_tableoperations.php']);
}

?>